local MAJOR_VERSION = "Threat-1.0"
local MINOR_VERSION = tonumber(("$Revision: 39990 $"):match("%d+"))

if MINOR_VERSION > _G.ThreatLib_MINOR_VERSION then
	_G.ThreatLib_MINOR_VERSION = MINOR_VERSION
end

local _, c = _G.UnitClass("player")
if c ~= "WARRIOR" then return end 

ThreatLib_funcs[#ThreatLib_funcs+1] = function()

local _G = _G
local max = _G.max
local tonumber = _G.tonumber
local type = _G.type
local select = _G.select
local GetTalentInfo = _G.GetTalentInfo
local GetShapeshiftForm = _G.GetShapeshiftForm
local AceLibrary = _G.AceLibrary

local ThreatLib = _G.ThreatLib

local BS = AceLibrary("Babble-Spell-2.2")
local Aura = AceLibrary("SpecialEvents-Aura-2.0")
local Warrior = ThreatLib:GetModule("ClassCore"):NewModule(c)

local threatValues = {
	["sunder"] 			= {nil, nil, nil, nil, 260, 301},
	["shieldBash"]		= {nil, nil, 180, 230},
	["revenge"]			= {nil, nil, nil, nil, nil, nil, nil, 201},
	["devastate"] 		= {101, 101, 101},
	["heroicStrike"]	= {nil, nil, nil, nil, nil, nil, nil, 145, 173, 196, 220},
	["shieldSlam"]		= {nil, nil, nil, 250, 286, 307},
	["cleave"]			= {nil, nil, nil, nil, 100, 130},
	["thunderclap"]		= 1.75,
	["execute"]			= 1.25,
	["hamstring"]		= {nil, nil, nil, 181},
	["mockingBlow"]		= {nil, nil, nil, nil, nil, 290},
	["disarm"]			= 104,
	["battleShout"]		= {nil, nil, nil, nil, nil, nil, 30, 69},
	["demoShout"]		= {nil, nil, nil, nil, nil, nil, 56},
}
	
function Warrior:ClassInit()
	self.className = "Warrior"

	-- Scan talents
	-- Defiance

	self:GetStanceThreatMod()
	
	-- CastHandlers should return true if they modified the threat level, false if they don't
	self.CastHandlers[BS["Taunt"]] 				= self.Taunt
	self.CastHandlers[BS["Sunder Armor"]] 		= self.SunderArmor
	self.CastHandlers[BS["Heroic Strike"]]		= self.HeroicStrike
	self.CastHandlers[BS["Shield Bash"]]		= self.ShieldBash
	self.CastHandlers[BS["Shield Slam"]]		= self.ShieldSlam
	self.CastHandlers[BS["Revenge"]]			= self.Revenge
	self.CastHandlers[BS["Mocking Blow"]]		= self.MockingBlow
	self.CastHandlers[BS["Commanding Shout"]]	= function() self:AddThreat(68) end
	self.CastHandlers[BS["Hamstring"]]			= self.Hamstring
	self.CastHandlers[BS["Battle Shout"]]		= self.BattleShout
	self.CastHandlers[BS["Demoralizing Shout"]]	= self.DemoShout
	self.CastMissHandlers[BS["Demoralizing Shout"]]	= self.DemoShoutMiss
	
	self.CastHandlers[BS["Disarm"]]				= self.Disarm
	self.CastHandlers[BS["Devastate"]]			= self.Devastate
	
	-- ThreatMods should return the modifications (if any) to the threat passed in
	self.abilityThreatMods[BS["Thunder Clap"]]	= self.Thunderclap
	self.abilityThreatMods[BS["Execute"]]		= self.Execute
	
	-- Set names don't need to be localized.
	self.itemSets = {
		["Might"] = { 16866, 16867, 16868, 16862, 16864, 16861, 16865, 16863 }
	}

	self:RegisterEvent("UPDATE_SHAPESHIFT_FORM", "GetStanceThreatMod" )
end

function Warrior:ScanTalents()
	local rank = _G.select(5, GetTalentInfo(3, 9))
	self.defianceMod = 1 + (0.05 * rank)
end

function Warrior:GetLinearThreatGuess(index, rank)
	if type(rank) == "string" then
		local ranknum = tonumber(rank:match("%d+"))
		rank = ranknum
	end
	local threatValues_index = threatValues[index]
	local v = threatValues_index[rank]
	if not v then
		local m = threatValues_index[#threatValues_index]
		v = (m / #threatValues_index) * rank
	end
	return v
end

function Warrior:GetStanceThreatMod()
	if GetShapeshiftForm() == 2 then
		self.passiveThreatModifiers = 1.3 * self.defianceMod
	else
		self.passiveThreatModifiers = 0.8
	end
	self.totalThreatMods = nil		-- Needed to recalc total mods
end

function Warrior:SunderArmor(rank)
	local sunderMod = 1.0
	if self:getWornSetPieces("Might") >= 8 then	
		sunderMod = 1.15;
	end
	local ranknum = tonumber(rank:match("%d+"))
	local threat = self:GetLinearThreatGuess("sunder", ranknum)
	self:AddTargetThreat(threat * sunderMod * self:threatMods())
end

function Warrior:Taunt()
	if UnitExists("targettarget") then
		local myThreat = ThreatLib:GetThreat(UnitName("player"), UnitName("target"))
		local theirThreat = ThreatLib:GetThreat(UnitName("targettarget"), UnitName("target"))
		if myThreat >= theirThreat then 
			return 
		end
		local globalThreat = ThreatLib:GetThreat(UnitName("player"), "_GLOBAL_")
		self:SetTargetThreat(theirThreat - globalThreat)
	else
		-- Assume we get max threat? This seems dangerous, but we need it to apply taunt in situations where the mob is sheeped/stunned/shackled/anything else that prevents a target
		local maxThreat = ThreatLib:GetMaxThreatOnTarget(UnitName("target")) 
		local globalThreat = ThreatLib:GetThreat(UnitName("player"), "_GLOBAL_")
		self:SetTargetThreat(maxThreat - globalThreat)
	end	
end

function Warrior:HeroicStrike(rank)
	local threat = self:GetLinearThreatGuess("heroicStrike", rank)
	self:AddTargetThreat(threat * self:threatMods())
end

function Warrior:ShieldBash(rank)
	local threat = self:GetLinearThreatGuess("shieldBash", rank)
	self:AddTargetThreat(threat * self:threatMods())
end

function Warrior:ShieldSlam(rank)
	local threat = self:GetLinearThreatGuess("shieldSlam", rank)
	self:AddTargetThreat(threat * self:threatMods())
end

function Warrior:Revenge(rank)
	local threat = self:GetLinearThreatGuess("revenge", rank)
	self:AddTargetThreat(threat * self:threatMods())
end

function Warrior:MockingBlow(rank)
	local threat = self:GetLinearThreatGuess("mockingBlow", rank)
	self:AddTargetThreat(threat * self:threatMods())
end

function Warrior:Hamstring(rank)
	local threat = self:GetLinearThreatGuess("hamstring", rank)
	self:AddTargetThreat(threat * self:threatMods())
end

function Warrior:Thunderclap(amt)
	return amt * threatValues.thunderclap
end

function Warrior:Execute(amt)
	return amt * threatValues.execute
end

function Warrior:Disarm()
	return threatValues.disarm
end

function Warrior:BattleShout(rank)
	local threat = self:GetLinearThreatGuess("battleShout", rank)
	self:AddThreat(threat * self:threatMods())
end

local function applyDemo(self, target, threat)
	self:_addTargetThreat(threat, target)
end
function Warrior:DemoShout(rank)
	local threat = self:GetLinearThreatGuess("demoShout", rank) * self:threatMods()
	local ct = 0
	for target in pairs(self.targetThreat) do
		ct = ct + 1
	end
	
	for target in pairs(self.targetThreat) do
		self:ScheduleEvent("ThreatLib-Warrior-DemoShout-" .. target, applyDemo, 0.5, self, target, threat / ct)
	end
end

function Warrior:DemoShoutMiss(target)
	if self:IsEventScheduled("ThreatLib-Warrior-DemoShout-" .. target) then
		self:CancelScheduledEvent("ThreatLib-Warrior-DemoShout-" .. target)
	end
end

function Warrior:Devastate(rank)
	local threat = self:GetLinearThreatGuess("devastate", rank)
	for buffName, index, applications, texture, rank in Aura:BuffIter("player") do
		if buffName == BS["Sunder Armor"] then
			threat = threat * applications
			break
		end
	end	
	self:AddTargetThreat(threat * self:threatMods())
end

table.insert(ThreatLib.UpvalueFixers, function(lib)
	ThreatLib = lib
	Warrior = ThreatLib:GetModule("ClassCore"):GetModule(c)
end)

end
